// HFSIterator.h
//	Copyright:	 1994 - 1998 by Apple Computer, Inc., all rights reserved.

// a tool for efficiently implementing IADocIterator on HFS file systems

#pragma once
#ifndef HFSIterator_h
#define HFSIterator_h

#pragma import on

#if PRAGMA_STRUCT_ALIGN
	#pragma options align=power
#endif

#include "IACommon.h"

#include <Files.h>
#include <Time.h>

#pragma IA_BEGIN_EXPORTS

struct DirectoryInfo {
	long		id;								// the id
	short		length;							// the number of files
private:
	void*		operator new(size_t size);		// stack or array allocate only
};

//  progress report support
//typedef void TProgressFn(float percent, void* data);

class HFSIterator : public IAObject {
public:
						HFSIterator(short vRefNum, long rootDirId = 2, 
						TProgressFn*	progressFn = NULL,
						clock_t			progressFreq = 0xFFFFFFFFul, // never
						void*			appData = NULL);
										
						~HFSIterator();
	bool				Increment();

	CInfoPBRec* 		GetPBRec() const {return pb;}
	
	// Accessors needed to override Increment()
	DirectoryInfo*		GetDirInfos() const {return dirInfos;}
	long				GetDirCount() const {return dirCount;}
	
	uint32				GetDir() const {return dir;}
	void				SetDir(uint32 newValue) {dir = newValue;}
	
	short				GetDirIndex() const {return dirIndex;}
	void				SetDirIndex(short newValue) {dirIndex = newValue;}

	TProgressFn*	GetProgressFn() const 	{return fProgressFn;}
	void*			GetProgressData() const 	{return fProgressData;}
	clock_t			GetProgressFreq() const 	{return fProgressFreq;}
	
protected:
	
	void				CollectDirInfo();

private:
	void				CollectDirInfoInternal(uint32* dirInfosLen);
	
	TProgressFn*		fProgressFn;
	void*				fProgressData;
	clock_t				fProgressFreq;

	CInfoPBRec*			pb;					// info about current file
	DirectoryInfo*		dirInfos;			// sorted array of directory ids for current volume
	long				dirCount;			// length of dirInfos
	uint32				dir;				// index in dirInfos of directory being processed
	short				dirIndex;			// index into directory being processed

};

#pragma IA_END_EXPORTS

#if PRAGMA_STRUCT_ALIGN
	#pragma options align=reset
#endif

#pragma import reset

#endif
