// HFSTextFolderCorpus.h
//	Copyright:	 1994 - 1998 by Apple Computer, Inc., all rights reserved.

// A corpus implementation for all the text files under a root HFS folder.

#pragma once
#ifndef HFSTextFolderCorpus_h
#define HFSTextFolderCorpus_h

#pragma import on

#if PRAGMA_STRUCT_ALIGN
	#pragma options align=power
#endif

#include "HFSCorpus.h"
#include "HFSIterator.h"

#pragma IA_BEGIN_EXPORTS

const uint32	HFSFolderCorpusType = 'HTF1';

class HFSTextFolderCorpus : public HFSCorpus {
public:	
				HFSTextFolderCorpus(uint32 type = HFSFolderCorpusType);
				HFSTextFolderCorpus(short vRef, long rootDirID, uint32 type = HFSFolderCorpusType);
				HFSTextFolderCorpus(StringPtr rootDirPath, uint32 type = HFSFolderCorpusType);

	virtual ~HFSTextFolderCorpus() {};
	// IACorpus methods
	IADoc*		GetProtoDoc();
	IADocIterator* GetDocIterator();
	
	IADocIterator* GetDocIterator(TProgressFn*	progressFn,
						clock_t			progressFreq, // never
						void*			appData);
	
	void SetVolumeRefNum (short vrn) {	vRefNum = vrn;}
	short GetVolumeRefNum () const {return vRefNum;}
	
	void SetRootDirID(long rootdirid) {rootDirId = rootdirid;}
	long GetRootDirID() const {return rootDirId;}
	 
protected:
	IABlockSize	InitialSize();
	void		Initializing(IAOutputBlock* output);
	void		Opening(IAInputBlock* input);
	IABlockSize	UpdateSize();
	void		Updating(IAOutputBlock* output);
private:
				HFSTextFolderCorpus(HFSTextFolderCorpus& fc);
	short		vRefNum;
	long		rootDirId;

};

class HFSTextFolderDoc : public HFSDoc {
public:
				HFSTextFolderDoc(HFSTextFolderCorpus* corpus, long dirID, const StringPtr name, long date);
				HFSTextFolderDoc() : HFSDoc() {}
	virtual ~HFSTextFolderDoc() {};
	IAStorable*	DeepCopy() const;
	IABlockSize	StoreSize() const;
	void		Store(IAOutputBlock* output) const;
	IAStorable*	Restore(IAInputBlock* input) const;

	bool		LessThan(const IAOrderedStorable* neighbor) const;
	bool		Equal(const IAOrderedStorable* neighbor) const;
	
	void	SetModDate (long mDate) {modDate = mDate;}
	long	GetModDate () const {return modDate;}

protected:
	void		DeepCopying(const IAStorable* source);
	void		Restoring(IAInputBlock* input, const IAStorable* proto);
private:
				HFSTextFolderDoc(HFSDoc& fd);
	long		modDate;
};

#pragma IA_END_EXPORTS

#if PRAGMA_STRUCT_ALIGN
	#pragma options align=reset
#endif

#pragma import reset

#endif
