// SentCorpus.h
//	Copyright:	 1994 - 1998 by Apple Computer, Inc., all rights reserved.
// A corpus that contains just one document. It treats each sentence as a document.
// Used for document summarization.

#pragma once
#ifndef SentCorpus_h
#define SentCorpus_h

#pragma import on

#if PRAGMA_STRUCT_ALIGN
	#pragma options align=power
#endif

#include "StringCorpus.h"

#pragma IA_BEGIN_EXPORTS

const uint32	IASentenceCorpusType = 'Snt0';

class RankedHit;

class SentenceDoc : public StringDoc {
public:
		SentenceDoc(byte* txt, uint32 len, uint32 sentenceNumber, uint32 paragraphNumber,
					uint32 rank=0, RankedHit* hit = NULL);
		SentenceDoc() : StringDoc(), fSentenceNumber(0), fRank(0), fParagraphNumber(0), fHit(NULL) {}
		virtual		~SentenceDoc();

		IAStorable*	DeepCopy() const;
		uint32		StoreSize() const;
		void		Store(IAOutputBlock* output) const;
		IAStorable*	Restore(IAInputBlock* input) const;

		bool		LessThan(const IAOrderedStorable* neighbor) const;
		bool		Equal(const IAOrderedStorable* neighbor) const;
		

		void 		SetRank(uint32 rank) {fRank = rank;}
		void		SetRankedHit(RankedHit* rh) {fHit = rh;}
		
		uint32		GetSentenceNumber() const {return fSentenceNumber;}
		uint32 		GetRank() const {return fRank;}
		uint32		GetParagraphNumber() const {return fParagraphNumber;}
		RankedHit*	GetRankedHit() const {return fHit;}
		
private:
		uint32		fSentenceNumber;
		uint32		fRank;
		uint32		fParagraphNumber;
		RankedHit*	fHit;
};


class SentenceCorpus : public IACorpus {
public:	
				SentenceCorpus() : IACorpus(IASentenceCorpusType) {};
//				SentenceCorpus(SentenceCorpus& fc) : IACorpus(fc) {};
			virtual 	~SentenceCorpus() {}; 
				IADoc*	GetProtoDoc();
				IADocText*	GetDocText(const IADoc* doc);
};

#pragma IA_END_EXPORTS

#if PRAGMA_STRUCT_ALIGN
	#pragma options align=reset
#endif

#pragma import reset

#endif

